/*
Copyright (C) 2015 Felipe Izzo
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "quakedef.h"
#include "errno.h"

#include <3ds.h>
#include "ctr.h"

u8 isN3DS;

#define TICKS_PER_SEC 268123480.0

qboolean		isDedicated;

u64 initialTime = 0;
int hostInitialized = 0;

/*
===============================================================================

FILE IO

===============================================================================
*/

#define MAX_HANDLES             10
FILE    *sys_handles[MAX_HANDLES];

int             findhandle (void)
{
	int             i;

	for (i=1 ; i<MAX_HANDLES ; i++)
		if (!sys_handles[i])
			return i;
	Sys_Error ("out of handles");
	return -1;
}

/*
================
filelength
================
*/
int filelength (FILE *f)
{
	int             pos;
	int             end;

	pos = ftell (f);
	fseek (f, 0, SEEK_END);
	end = ftell (f);
	fseek (f, pos, SEEK_SET);

	return end;
}

int Sys_FileOpenRead (char *path, int *hndl)
{
	FILE    *f;
	int             i;

	i = findhandle ();

	f = fopen(path, "rb");
	if (!f)
	{
		*hndl = -1;
		return -1;
	}
	sys_handles[i] = f;
	*hndl = i;

	return filelength(f);
}

int Sys_FileOpenWrite (char *path)
{
	FILE    *f;
	int             i;

	i = findhandle ();

	f = fopen(path, "wb");
	if (!f)
		Sys_Error ("Error opening %s: %s", path,strerror(errno));
	sys_handles[i] = f;

	return i;
}

void Sys_FileClose (int handle)
{
	fclose (sys_handles[handle]);
	sys_handles[handle] = NULL;
}

void Sys_FileSeek (int handle, int position)
{
	fseek (sys_handles[handle], position, SEEK_SET);
}

int Sys_FileRead (int handle, void *dest, int count)
{
	return fread (dest, 1, count, sys_handles[handle]);
}

int Sys_FileWrite (int handle, void *data, int count)
{
	return fwrite (data, 1, count, sys_handles[handle]);
}

int     Sys_FileTime (char *path)
{
	FILE    *f;

	f = fopen(path, "rb");
	if (f)
	{
		fclose(f);
		return 1;
	}

	return -1;
}

void Sys_mkdir (char *path)
{
}


/*
===============================================================================

SYSTEM IO

===============================================================================
*/

void Sys_MakeCodeWriteable (unsigned long startaddr, unsigned long length)
{
}


void Sys_Error (char *error, ...)
{
	va_list         argptr;

	printf ("Sys_Error: ");
	va_start (argptr,error);
	vprintf (error,argptr);
	va_end (argptr);
	printf ("\n");
	printf("Press START to exit");
	while(1){
		hidScanInput();
		u32 kDown = hidKeysDown();
		if (kDown & KEY_START)
			break;
	}
	Host_Shutdown();
	exit(1);
}

void Sys_Printf (char *fmt, ...)
{
	if(hostInitialized)
		return;

	va_list         argptr;

	va_start (argptr,fmt);
	vprintf (fmt,argptr);
	va_end (argptr);
}

void Sys_Quit (void)
{
	Host_Shutdown();
	gfxExit();
	exit (0);
}

double Sys_FloatTime (void)
{
	if(!initialTime){
		initialTime = svcGetSystemTick();
	}
	u64 curTime = svcGetSystemTick();
	return (curTime - initialTime)/TICKS_PER_SEC;
}

char *Sys_ConsoleInput (void)
{
	return NULL;
}

void Sys_Sleep (void)
{
}

char CTR_GetKey(u32 down, u32 held){     
	char letters[72] = {'1','2','3','4','5','6','7','8','9','0','c','d',
						 'e','f','g','h','i','j','k','l','m','n','o','p',
						 'q','r','s','t','u','v','w','z','y','x','a','b',		
						 '-','=',',','.','/',';','\'','[',']','`',K_SPACE,K_BACKSPACE,	
						 K_F1,K_F2,K_F3,K_F4,K_F5,K_F6,K_F7,K_F8,K_F9,K_F10,K_F11,K_F12,
						 K_AUX1,K_AUX2,K_AUX3,K_AUX4,K_AUX5,K_AUX6,K_AUX7,K_AUX8,K_LEFTARROW,K_UPARROW,K_RIGHTARROW,K_DOWNARROW}; 
    
    char chr = '\0';      
    
    u32 kSet = 0; //Set of characters
	u32 kPos = 0; //Position 1 is up 2 is up right
	
	u32 kDown = down;
	u32 kHeld = held;
	
    
    //Choose character set
    if(kHeld & KEY_R)
		 kSet = 1;
	else kSet = 0;
	
	//Choose direction
	if (kHeld & KEY_DUP){
		if (kHeld & KEY_DLEFT) kPos = 7;
		else if (kHeld & KEY_DRIGHT) kPos = 1;
		else kPos = 0;
	} else if(kHeld & KEY_DDOWN){
		if (kHeld & KEY_DLEFT) kPos = 5;
		else if(kHeld & KEY_DRIGHT) kPos = 3;
		else kPos = 4;
	}else if(kHeld & KEY_DLEFT) kPos = 6;		
	else if(kHeld & KEY_DRIGHT) kPos = 2;
	else kPos = 8;
	
	//actual button pressing
	if (kDown & KEY_Y)
       chr = letters[(36 * kSet) + (4 * kPos) + 0];
    else if (kDown & KEY_X)
       chr = letters[(36 * kSet) + (4 * kPos) + 1];
    else if (kDown & KEY_A)
       chr = letters[(36 * kSet) + (4 * kPos) + 2];
    else if (kDown & KEY_B)
       chr = letters[(36 * kSet) + (4 * kPos) + 3];
    else chr = '0';
        
    return chr;
}

void CTR_KeyDown(u32 down, u32 held){
	if( down & KEY_SELECT)
		Key_Event(K_ESCAPE, true);
	if( down & KEY_START)
		Key_Event(K_ENTER, true);
	if( down & KEY_L)
		Key_Event(K_SHIFT, true);
	if( down & KEY_Y)
		Key_Event(CTR_GetKey(down, held), true);
	if( down & KEY_X)
		Key_Event(CTR_GetKey(down, held), true);
	if( down & KEY_B)
		Key_Event(CTR_GetKey(down, held), true);
	if( down & KEY_A)
		Key_Event(CTR_GetKey(down, held), true);
}

void CTR_KeyUp(u32 up, u32 held){
	if( up & KEY_SELECT)
		Key_Event(K_ESCAPE, false);
	if( up & KEY_START)
		Key_Event(K_ENTER, false);
	if( up & KEY_L)
		Key_Event(K_SHIFT, false);	
	if( up & KEY_Y)
		Key_Event(CTR_GetKey(up, held), false);
	if( up & KEY_X)
		Key_Event(CTR_GetKey(up, held), false);
	if( up & KEY_A)
		Key_Event(CTR_GetKey(up, held), false);
	if( up & KEY_B)
		Key_Event(CTR_GetKey(up, held), false);
}

void Sys_SendKeyEvents (void)
{
	hidScanInput();
	u32 kDown = hidKeysDown();
	u32 kHeld = hidKeysHeld();
	u32 kUp = hidKeysUp();
	if(kDown)
		CTR_KeyDown(kDown, kHeld);
	if(kUp)
		CTR_KeyUp(kUp, kHeld);
	//debug the keys
	if( kDown & KEY_ZR)
		Key_ClearStates();
}

void Sys_HighFPPrecision (void)
{
}

void Sys_LowFPPrecision (void)
{
}

//=============================================================================

int main (int argc, char **argv)
{
	float		time, oldtime;
	APT_CheckNew3DS(NULL, &isN3DS);
	if(isN3DS)
		osSetSpeedupEnable(true);

	gfxInit(GSP_RGB565_OES,GSP_RGB565_OES,false);
	hidInit();
	gfxSetDoubleBuffering(GFX_TOP, false);
	gfxSetDoubleBuffering(GFX_BOTTOM, false);
	gfxSet3D(false);
	consoleInit(GFX_BOTTOM, NULL);
	static quakeparms_t    parms;

	parms.memsize = 16*1024*1024;
	parms.membase = malloc (parms.memsize);
	parms.basedir = ".";

	COM_InitArgv (argc, argv);

	parms.argc = com_argc;
	parms.argv = com_argv;
	Host_Init (&parms);
	hostInitialized = true;
	ctrDrawTouchOverlay();
	//Sys_Init();
	oldtime = Sys_FloatTime() -0.1;
	while (1)
	{
		time = Sys_FloatTime();
		Host_Frame (time - oldtime);
		oldtime = time;
	}
	gfxExit();
	return 0;
}
